package com.altinity.opsview.jfreechart;

/**
 * Copyright (C) 2008 Opsera Limited. All rights reserved
 *
 * This file is part of Opsview
 * 
 * Opsview is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * Opsview is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Opsview; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301  USA
 * 
 */

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.geom.Rectangle2D;
import java.util.Date;
import java.util.Iterator;
import java.util.List;

import org.jfree.chart.JFreeChart;
import org.jfree.chart.LegendItem;
import org.jfree.chart.LegendItemCollection;
import org.jfree.chart.axis.DateAxis;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.plot.ValueMarker;
import org.jfree.chart.plot.XYPlot;

import com.altinity.opsview.odw.bc.ODWReportBC;
import com.excira.jfreechart.WeekChart;
import com.excira.jfreechart.XYPoint;

public class ThresholdWeekChart extends WeekChart {
	
	private double warning = 0d;
	private double critical = 0d;
	
	private Date startDate;
	private Date endDate;
	
	/**
	 * 
	 */
	public ThresholdWeekChart(double warning, double critical, Date startDate, Date endDate) {
		
		this.warning = warning;
		this.critical = critical;
		
		this.startDate = startDate;
		this.endDate = endDate;
	}
	
	/**
	 * 
	 */
	protected void setDomainAxis(DateAxis axis, List xyPoints) {
		
		axis.setAutoRange(false);
		axis.setRange(startDate, endDate);
	}
	
	/**
	 * 
	 */
	protected void setRangeAxis(NumberAxis axis, List xyPoints) {
		
		axis.setAutoRange(false);
		
		if ( xyPoints.isEmpty() ) {
			axis.setRange(0, 100);
			return;
		}
		
		// Get the min and max for range
		// Set initial min and max to be value
		// of first xyPoint
		double min = ((XYPoint)xyPoints.get(0)).getY();
		double max = ((XYPoint)xyPoints.get(0)).getY();
		
		Iterator i = xyPoints.iterator();
		while ( i.hasNext() ) {
			
			XYPoint xyPoint = (XYPoint)i.next();
			double y = xyPoint.getY();

			if ( min > y )
				min = y;
			
			if ( max < y )
				max = y;	
		}
		
		// Now get minimum out of warning threshold 
		// and min
		if ( critical > 0d && critical < min )
			min = critical;
		
		// Add buffer to min value
		min = min - (min * 0.05);
		
		axis.setRange(min, 100);
	}
	
	/**
	 * 
	 */
	public double getWarning() {
		return warning;
	}
	
	/**
	 * 
	 */
	public void setWarning(double warning) {
		this.warning = warning;
	}
	
	/**
	 * 
	 */
	protected void addRangeMarker(XYPlot plot) {
		
		// Add range marker at warning and 
		// critical levels if necessary
		if ( warning > 0d ) {
			
			ValueMarker warningMarker = new ValueMarker(warning);
			warningMarker.setPaint(getYellow());
			warningMarker.setStroke(new BasicStroke(1.1f));
			plot.addRangeMarker(warningMarker);
		}
		
		if ( critical > 0d ) {
			
			ValueMarker criticalMarker = new ValueMarker(critical);
			criticalMarker.setPaint(getRed());
			criticalMarker.setStroke(new BasicStroke(1.1f));
			plot.addRangeMarker(criticalMarker);
		}
	}

	/**
	 * 
	 */
	public double getCritical() {
		return critical;
	}

	/**
	 * 
	 */
	public void setCritical(double critical) {
		this.critical = critical;
	}
	
	/**
	 * 
	 */
	protected void updateLegend(JFreeChart chart) {
	
		if ( warning == 0d && critical == 0d )
			return; 
		
		((XYPlot)chart.getPlot()).setFixedLegendItems(createLegendItems());	
	}
	
	/**
	 * 
	 */
	private LegendItemCollection createLegendItems() {
		
		LegendItemCollection collection = new LegendItemCollection();
		
		// Add legend item for warning threshold
		LegendItem warning = 
			new LegendItem("1st availability threshold", "-", null, null, 
					new Rectangle2D.Double(-0.0, -0.0, 16.0, 2.0), getYellow());
		collection.add(warning);
		
		// Add legend item for critical threshold
		LegendItem red =
			new LegendItem("2nd availability threshold", "-", null, null, 
					new Rectangle2D.Double(-0.0, -0.0, 16.0, 2.0), getRed());
		collection.add(red);
		
		return collection;
	}
	
	/**
	 * 
	 */
	private Color getYellow() {
		
		Color yellow = 
			new Color(ODWReportBC.RGB_WARNING[0],
						ODWReportBC.RGB_WARNING[1],
						ODWReportBC.RGB_WARNING[2]);
		
		return yellow;
	}
	
	/**
	 * 
	 */
	private Color getRed() {
		
		Color red = 
			new Color(ODWReportBC.RGB_CRITICAL[0],
						ODWReportBC.RGB_CRITICAL[1],
						ODWReportBC.RGB_CRITICAL[2]);
		
		return red;
	}
	
	
	
	/**
	 * 
	 */

}
