package com.altinity.opsview.odw.dao;

/**
 * Copyright (C) 2008 Opsera Limited. All rights reserved
 *
 * This file is part of Opsview
 * 
 * Opsview is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * Opsview is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Opsview; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301  USA
 * 
 */

import java.sql.ResultSet;
import java.sql.SQLException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import org.springframework.dao.EmptyResultDataAccessException;
import org.springframework.jdbc.core.namedparam.BeanPropertySqlParameterSource;
import org.springframework.jdbc.core.simple.ParameterizedRowMapper;

import com.altinity.opsview.odw.ReportConstants;
import com.altinity.opsview.odw.exception.NoDescriptionFoundForHostException;
import com.altinity.opsview.odw.exception.NoPriorStatusFoundForHost;
import com.altinity.opsview.odw.exception.ReportException;
import com.altinity.opsview.odw.exception.UnableToDetermineHourSummaryForHostException;
import com.altinity.opsview.odw.model.AvailabilityModel;
import com.altinity.opsview.odw.model.HostGroup;
import com.altinity.opsview.odw.model.HourSummary;
import com.altinity.opsview.odw.model.PerformanceData;
import com.altinity.opsview.odw.model.ServiceCheck;
import com.altinity.opsview.odw.model.ServiceCheckKey;
import com.altinity.opsview.odw.model.StateHistory;
import com.altinity.opsview.odw.model.WeeklyServiceAvailability;
import com.altinity.opsview.odw.reports.ReportCriteria;
import com.excira.jfreechart.BarData;
import com.excira.jfreechart.DateChartData;
import com.excira.jfreechart.XYPoint;
import com.excira.utils.DateUtils;

public class ODWReportDAO extends ODWDAO {

	/**
	 * 
	 */
	public Date getDataStartDate(Date startDate, Date endDate) throws ReportException {
		
		// Find the first data load in the reporting period
		StringBuffer query = new StringBuffer();
		query.append("SELECT period_start_timev ");
		query.append("FROM dataloads ");
		query.append("WHERE period_start_timev >= :startTime ");
		query.append("and period_end_timev <= :endTime ");
		query.append("and status = 'success' ");
		query.append("order by period_start_timev asc ");
		query.append("limit 1");
		
		// Get start and end time in seconds
		Map <String, Long> params = new HashMap<String, Long>(2);
		params.put("startTime", (startDate.getTime() / 1000));
		params.put("endTime", (endDate.getTime() / 1000));
		
		Date dataStartDate = null;
		
		try {
			
			dataStartDate = getJdbcTemplate().queryForObject(query.toString(), new SimpleIntToDateMapper(), params);
		}
		catch(EmptyResultDataAccessException e) {
			
			SimpleDateFormat format = new SimpleDateFormat(ReportConstants.DATE_FORMAT);
			throw new ReportException("No data loaded for report period " 
					+ format.format(startDate) + " to " + format.format(endDate));
		}
		return dataStartDate;
	}
	
	/**
	 * 
	 */
	public Date getDataEndDate(Date startDate, Date endDate) throws ReportException {
		
		// First increment the end date by one day
		// and make sure time element is midnight
		Calendar cal = Calendar.getInstance();
		cal.setTime(endDate);
		cal.add(Calendar.DATE, 1);
		cal.set(Calendar.HOUR, 0);
		cal.set(Calendar.MINUTE, 0);
		cal.set(Calendar.SECOND, 0);
		
		// Now generate a query to find the last data upload before
		// midnight of the day after the end date
		StringBuffer query = new StringBuffer();
		query.append("SELECT period_end_timev ");
		query.append("FROM dataloads ");
		query.append("WHERE period_start_timev >= :startTime ");
		query.append("and period_end_timev < :endTime ");
		query.append("and status = 'success' ");
		query.append("order by period_end_timev desc ");
		query.append("limit 1");
		
		// Get start and end time in seconds
		Map <String, Long> params = new HashMap<String, Long>(2);
		params.put("startTime", (startDate.getTime() / 1000));
		params.put("endTime", (endDate.getTime() / 1000));
		
		Date dataEndDate = null;
		
		try {
			
			dataEndDate = getJdbcTemplate().queryForObject(query.toString(), new SimpleIntToDateMapper(), params);
		}
		catch(EmptyResultDataAccessException e) {
			
			SimpleDateFormat format = new SimpleDateFormat(ReportConstants.DATE_FORMAT);
			throw new ReportException("No data loaded for report period " 
					+ format.format(startDate) + " to " + format.format(endDate));
		}
		return dataEndDate;
	}

	/**
	 * 
	 */
	public List<PerformanceData> queryPerformanceData(String hostName, String serviceCheckName, String perfLabel,
			Date startDate, Date endDate) {

		PerformanceDataMappingQuery query = new PerformanceDataMappingQuery(dataSource);
		Object[] params = new Object[5];
		params[0] = hostName;
		params[1] = serviceCheckName;
		params[2] = perfLabel;
		params[3] = DateUtils.format(startDate);
		params[4] = DateUtils.format(endDate);

		return query.execute(params);
	}

	/**
	 * 
	 */
	public HourSummary queryHourSummary(String hostName, String serviceCheckName, Date startDate)
			throws UnableToDetermineHourSummaryForHostException {

		List<HourSummary> summaries = null;
		try {
			
			HourSummaryMappingQuery query = new HourSummaryMappingQuery(dataSource);
			Object[] params = new Object[3];
			params[0] = hostName;
			params[1] = serviceCheckName;
			params[2] = DateUtils.format(startDate);

			summaries = query.execute(params);

			// Throw an exception if no summary is found
			if (summaries.isEmpty())
				throw new RuntimeException("No state history hour summary found "
						+ "in ODWReportBC.getChartData for host '" + hostName + "', service check '" + serviceCheckName
						+ "' " + "and start date '" + DateUtils.format(startDate) + "'");
			// Throw an exception if more than one summary is found
			else if (summaries.size() > 1)
				throw new RuntimeException("More than one state history "
						+ "hour summary found in ODWReportBC.getChartData for host '" + hostName + "', service check '"
						+ serviceCheckName + "' " + "and start date '" + DateUtils.format(startDate) + "'");
		} catch (Exception e) {
			System.out.println("Error in ODWReportDAO.queryHourSummary");
			e.printStackTrace();
			throw new UnableToDetermineHourSummaryForHostException();
		}
		return summaries.get(0);
	}

	/**
	 * 
	 */
	public List<StateHistory> queryStateHistory(String hostName, String serviceCheckName, Date startDate, Date endDate) {

		StateHistoryMappingQuery query = new StateHistoryMappingQuery(dataSource);
		Object[] params = new Object[4];
		params[0] = hostName;
		params[1] = serviceCheckName;
		params[2] = DateUtils.format(startDate);
		params[3] = DateUtils.format(endDate);

		return query.execute(params);
	}

	/**
	 * 
	 */
	public String queryPriorStatus(String hostName, String serviceCheckName, Date transitionDate)
			throws NoPriorStatusFoundForHost {

		Object priorStatus = null;

		try {
			priorStatus = getJdbcTemplate().queryForObject(QueryBuilder.getQueryForPriorStatus(), String.class,
					new Object[] { hostName, serviceCheckName, transitionDate });
		} catch (Exception e) {
			System.out.println("Error in ODWReportDAO.queryLastState");
			e.printStackTrace();
			throw new NoPriorStatusFoundForHost();
		}
		return (String) priorStatus;
	}

	/**
	 * @param hostName
	 * @return
	 * @throws NoDescriptionFoundForHostException
	 */
	public String queryHostDescription(String hostName) throws NoDescriptionFoundForHostException {

		Object description = null;

		try {
			description = getJdbcTemplate().queryForObject(QueryBuilder.getQueryForHostDescription(), String.class,
					new Object[] { hostName });
		} catch (Exception e) {
			System.out.println("Error in ODWReportDAO.queryHostDescription");
			e.printStackTrace();
			throw new NoDescriptionFoundForHostException();
		}
		return (String) description;
	}

	/**
	 * @param rc
	 * @return
	 */
	public AvailabilityModel getOverallAvailability(ODWReportConfigDAO configDao, ReportCriteria rc) {
		
		StringBuffer query = new StringBuffer();
		query.append("SELECT sum(seconds_ok)/sum(seconds_ok+seconds_not_ok)*100 availability, ");
		query.append("sum(seconds_not_ok_scheduled)/sum(seconds_ok+seconds_not_ok)*100 scheduled, ");
		query.append("sum(seconds_not_ok-seconds_not_ok_scheduled)/sum(seconds_ok+seconds_not_ok)*100 unscheduled ");
		query.append("FROM service_availability_hourly_summary, servicechecks, hosts ");
		query.append("WHERE CONVERT_TZ(start_datetime,'+00:00','SYSTEM') BETWEEN :dataStartDate and :dataEndDate ");
		query.append("and service_availability_hourly_summary.servicecheck = servicechecks.id ");
		query.append("and hosts.id = servicechecks.host ");
		query.append("and time(CONVERT_TZ(start_datetime,'+00:00','SYSTEM')) BETWEEN :dayIntervalStart and :dayIntervalEnd ");
		query.append("and hosts.hostgroup ");
		generateInList(query, configDao.getReportSections(rc));
		

		ParameterizedRowMapper<AvailabilityModel> mapper = new ParameterizedRowMapper<AvailabilityModel>() {
			public AvailabilityModel mapRow(ResultSet rs, int rowNum) throws SQLException {
				AvailabilityModel avail = new AvailabilityModel();
				avail.setAverageAvailablility(rs.getDouble(1));
				avail.setScheduledDowntime(rs.getDouble(2));
				avail.setUnscheduledDowntime(rs.getDouble(3));
				return avail;
			}
		};

		return getJdbcTemplate().queryForObject(query.toString(), mapper, new BeanPropertySqlParameterSource(rc));

	}
	
	/**
	 * @param rc
	 * @return
	 */
	public List<AvailabilityModel> getAvailabilityForHostGroups(ODWReportConfigDAO configDao, ReportCriteria rc) {
		
		// First get all host groups
		Map <String, HostGroup> hostGroupsByName = configDao.getHostGroupsForReport(rc.getReportName());
		
		// Finally, generate the availability for host groups query
		StringBuffer query = new StringBuffer();
		query.append("SELECT sum(seconds_ok)/sum(seconds_ok+seconds_not_ok)*100 availability, ");
		query.append("sum(seconds_not_ok_scheduled)/sum(seconds_ok+seconds_not_ok)*100 scheduled, ");
		query.append("sum(seconds_not_ok-seconds_not_ok_scheduled)/sum(seconds_ok+seconds_not_ok)*100 unscheduled, ");
		query.append("hosts.hostgroup hostgroup ");
		query.append("FROM service_availability_hourly_summary, servicechecks, hosts ");
		query.append("WHERE CONVERT_TZ(start_datetime,'+00:00','SYSTEM') BETWEEN :dataStartDate and :dataEndDate ");
		query.append("and service_availability_hourly_summary.servicecheck = servicechecks.id ");
		query.append("and hosts.id = servicechecks.host ");
		query.append("and time(CONVERT_TZ(start_datetime,'+00:00','SYSTEM')) BETWEEN :dayIntervalStart and :dayIntervalEnd ");
		query.append("and hosts.hostgroup ");
		generateInList(query, configDao.getReportSections(rc));
		query.append(" GROUP BY hostgroup ");
		query.append("ORDER BY hostgroup");

		// Execute the query and return a list of availability models
		return getJdbcTemplate().query(query.toString(), 
								new AvailabilityModelMapper(hostGroupsByName), 
								new BeanPropertySqlParameterSource(rc));

	}
	
	/**
	 * 
	 */
	private class AvailabilityModelMapper implements ParameterizedRowMapper<AvailabilityModel> {
		
		Map <String, HostGroup> hostGroups;
		
		/**
		 * 
		 */
		public AvailabilityModelMapper(Map <String, HostGroup> hostGroups) {
			
			this.hostGroups = hostGroups;
		}

		/**
		 * 
		 */
		public AvailabilityModel mapRow(ResultSet rs, int rowNum) throws SQLException {
			
			AvailabilityModel avail = new AvailabilityModel();
			avail.setAverageAvailablility(rs.getDouble("availability"));
			avail.setScheduledDowntime(rs.getDouble("scheduled"));
			avail.setUnscheduledDowntime(rs.getDouble("unscheduled"));
			
			// Get host group from map and add to availability model 
			String hostGroupName = rs.getString("hostgroup");
			HostGroup hostGroup = hostGroups.get(hostGroupName);
			avail.setHostGroup(hostGroup);
			
			// Add week and year if values are in result set
			if ( rs.getMetaData().getColumnCount() > 4 ) {
				avail.setWeek(rs.getString("week"));
				avail.setYear(rs.getString("year"));
			}
			
			return avail;
		}
	}
	
	/**
	 * @param rc
	 * @return
	 */
	public List <AvailabilityModel> getWeeklyAvailabilityForHostGroups(ODWReportConfigDAO configDao, ReportCriteria rc) {
		
		
		// First get all host groups
		Map <String, HostGroup> hostGroupsByName = configDao.getHostGroupsForReport(rc.getReportName());
		
		// Now generate the weekly availability for host groups query		
		StringBuffer query = new StringBuffer();
		query.append("SELECT sum(seconds_ok)/sum(seconds_ok+seconds_not_ok)*100 availability, ");
		query.append("sum(seconds_not_ok_scheduled)/sum(seconds_ok+seconds_not_ok)*100 scheduled, ");
		query.append("sum(seconds_not_ok-seconds_not_ok_scheduled)/sum(seconds_ok+seconds_not_ok)*100 unscheduled, ");
		query.append("hosts.hostgroup hostgroup, WEEK(CONVERT_TZ(start_datetime,'+00:00','SYSTEM'), 1) week, YEAR(CONVERT_TZ(start_datetime,'+00:00','SYSTEM')) year ");
		query.append("FROM service_availability_hourly_summary, servicechecks, hosts ");
		query.append("WHERE CONVERT_TZ(start_datetime,'+00:00','SYSTEM') BETWEEN  :dataStartDate and  :dataEndDate ");
		query.append("and service_availability_hourly_summary.servicecheck = servicechecks.id ");
		query.append("and hosts.id = servicechecks.host ");
		query.append("and time(CONVERT_TZ(start_datetime,'+00:00','SYSTEM')) BETWEEN  :dayIntervalStart  and  :dayIntervalEnd ");
		query.append("and hosts.hostgroup ");
		generateInList(query, configDao.getReportSections(rc));
		query.append(" GROUP BY hostgroup, week ORDER BY week, hostgroup");

		return getJdbcTemplate().query(query.toString(), 
				new AvailabilityModelMapper(hostGroupsByName),
				new BeanPropertySqlParameterSource(rc));

	}


	/**
	 * @param rc
	 * @return
	 */
	public List<XYPoint> getAverageAvailabilityTimeSeries(ODWReportConfigDAO configDao, ReportCriteria rc) {
		
		StringBuffer query = new StringBuffer();
		query.append("SELECT date(CONVERT_TZ(start_datetime,'+00:00','SYSTEM')) Date, ");
		query.append("sum(seconds_ok)/sum(seconds_ok+seconds_not_ok)*100 availability ");
		query.append("FROM service_availability_hourly_summary, servicechecks, hosts ");
		query.append("WHERE CONVERT_TZ(start_datetime,'+00:00','SYSTEM') BETWEEN :reportStartDateString and :reportEndDateString ");
		query.append("and service_availability_hourly_summary.servicecheck = servicechecks.id ");
		query.append("and hosts.id = servicechecks.host ");
		query.append("and time(CONVERT_TZ(start_datetime,'+00:00','SYSTEM')) BETWEEN :dayIntervalStart and :dayIntervalEnd ");
		query.append("and hosts.hostgroup ");
		generateInList(query, configDao.getReportSections(rc));
		query.append("GROUP BY Date ");
		query.append("ORDER BY Date");

		rc = new ReportCriteria(rc);

		Calendar cal = Calendar.getInstance();
		cal.setTime(rc.getReportStartDate());
		cal.add(Calendar.MONTH, -3);
		rc.setReportStartDate(cal.getTime());

		return getJdbcTemplate().query(query.toString(), new XYPointMapper(), new BeanPropertySqlParameterSource(rc));
	}
	

	
	/**
	 *
	 */
	public Map<HostGroup, List<XYPoint>> getAverageAvailabilityTimeSeriesForHostGroups(ODWReportConfigDAO configDao, ReportCriteria rc) {
		
		// First get all host groups
		Map <String, HostGroup> hostGroups = configDao.getHostGroupsForReport(rc.getReportName());
		
		// Now generate the availability query 
		StringBuffer query = new StringBuffer();
		query.append("SELECT date(CONVERT_TZ(start_datetime,'+00:00','SYSTEM')) Date, ");
		query.append("sum(seconds_ok)/sum(seconds_ok+seconds_not_ok)*100 availability FROM ");
		query.append("service_availability_hourly_summary, servicechecks, hosts ");
		query.append("WHERE CONVERT_TZ(start_datetime,'+00:00','SYSTEM') BETWEEN :reportStartDateString and :reportEndDateString ");
		query.append("and service_availability_hourly_summary.servicecheck = servicechecks.id ");
		query.append("and time(CONVERT_TZ(start_datetime,'+00:00','SYSTEM')) BETWEEN :dayIntervalStart and :dayIntervalEnd and hosts.id = servicechecks.host ");
		query.append("and hosts.hostgroup = :hostGroup ");
		query.append("GROUP BY Date");	
		

		// Setup the report criteria
		rc = new ReportCriteria(rc);
		Calendar cal = Calendar.getInstance();
		cal.setTime(rc.getReportStartDate());
		cal.add(Calendar.MONTH, -3);
		rc.setReportStartDate(cal.getTime());
		
		// Create a map to hold the points for each host group 
		Map <HostGroup, List<XYPoint>> pointsByHG = new LinkedHashMap<HostGroup, List<XYPoint>>();
		
		// Now get the points for each host group!
		for ( String hostGroupName : hostGroups.keySet() ) {
			
			HostGroup hostGroup = hostGroups.get(hostGroupName);
			rc.setHostGroup(hostGroupName);
			pointsByHG.put(hostGroup, 
					getJdbcTemplate().query(query.toString(), new XYPointMapper(), new BeanPropertySqlParameterSource(rc)));
		}
		
		return pointsByHG;
	}
	
	/**
	 * 
	 */
	private class XYPointMapper implements ParameterizedRowMapper<XYPoint> {
		
		public XYPoint mapRow(ResultSet rs, int rowNum) throws SQLException {
			
			XYPoint point = new XYPoint(rs.getDate(1), rs.getDouble(2));
			return point;
		}
	}

	/**
	 * 
	 */
	public List <List<BarData>> getAvailabilityTrendData(ODWReportConfigDAO configDao, ReportCriteria reportCriteria) {
		
		StringBuffer query = new StringBuffer();
		query.append("SELECT WEEK(CONVERT_TZ(start_datetime,'+00:00','SYSTEM'), 1) week, ");
		query.append("YEAR(CONVERT_TZ(start_datetime,'+00:00','SYSTEM')) year, ");
		query.append("sum(seconds_ok)/sum(seconds_ok+seconds_not_ok)*100 availability, ");
		query.append("sum(seconds_not_ok_scheduled)/sum(seconds_ok+seconds_not_ok)*100 scheduled, ");
		query.append("sum(seconds_not_ok-seconds_not_ok_scheduled)/sum(seconds_ok+seconds_not_ok)*100 unscheduled ");
		query.append("FROM service_availability_hourly_summary, servicechecks, hosts ");
		query.append("WHERE CONVERT_TZ(start_datetime,'+00:00','SYSTEM') BETWEEN :reportStartDateString and :reportEndDateString ");
		query.append("and service_availability_hourly_summary.servicecheck = servicechecks.id ");
		query.append("and hosts.id = servicechecks.host ");
		query.append("and time(CONVERT_TZ(start_datetime,'+00:00','SYSTEM')) BETWEEN :dayIntervalStart and :dayIntervalEnd ");
		query.append("and hosts.hostgroup ");
		generateInList(query, configDao.getReportSections(reportCriteria));
		query.append(" GROUP BY year, week");
		
		reportCriteria = new ReportCriteria(reportCriteria);

		Calendar cal = Calendar.getInstance();
		cal.setTime(reportCriteria.getReportStartDate());
		cal.add(Calendar.MONTH, -3);
		reportCriteria.setReportStartDate(cal.getTime());

		ParameterizedRowMapper<List<BarData>> mapper = new ParameterizedRowMapper<List<BarData>>() {
			
			public List <BarData>mapRow(ResultSet rs, int rowNum) throws SQLException {
				
				// Get date from week and year
				int weekOfYear = rs.getInt("week");
				int year = rs.getInt("year");
				Calendar cal = Calendar.getInstance();
				cal.set(Calendar.YEAR, year);
				cal.set(Calendar.WEEK_OF_YEAR, weekOfYear);
				DateUtils.adjustDateAccordingToWeekConfigToGetFirstDayOfWeek(cal, Calendar.MONDAY);
				
				DateChartData dateData = new DateChartData(cal.getTime());
				dateData.setPattern("dd-MMM-yy");
				
				List <BarData> data = new ArrayList<BarData>(3);
				
				BarData barData = new BarData(rs.getDouble("availability"), PERCENT_OK_INDEX, dateData);
				data.add(barData);
				
				barData = new BarData(rs.getDouble("scheduled"), PERCENT_SCHED_INDEX, dateData);
				data.add(barData);
				
				barData = new BarData(rs.getDouble("unscheduled"), PERCENT_UNSCHED_INDEX, dateData);
				data.add(barData);

				return data;
			}
		};

		List <List<BarData>> l = getJdbcTemplate().query(query.toString(), mapper, new BeanPropertySqlParameterSource(reportCriteria));
		
		return l; 
	}
	
	/**
	 * 
	 */
	public List<ServiceCheckKey> listSCKeysByKeywords(ReportCriteria rc, List <String> keywords) {
		
		StringBuffer sql = new StringBuffer();
		sql.append("select servicechecks.name name, nagios_object_id, ");
		sql.append("servicechecks.hostname hostName ");
		sql.append("from servicechecks ");
		sql.append("where ");
		boolean firstLike = true;
		for ( String keyword : keywords ) {

			// Append "and" between like 
			if ( firstLike )
				firstLike = false;
			else
				sql.append("and ");
			
			// Append keyword
			sql.append("keywords like '%,");
			sql.append(keyword);
			sql.append(",%' ");
		}
		sql.append("group by nagios_object_id");
		
		return getJdbcTemplate().query(sql.toString(), new ServiceCheckKeyMapper());
	}
	
	/**
	 * 
	 */
	public List<ServiceCheck> listServiceChecksByKey(ReportCriteria rc, ServiceCheckKey key) {
		
		StringBuffer sql = new StringBuffer();
		sql.append("select servicechecks.id, ");
		
		if ( rc.isUseHostNameServiceCheckLabel() )
			sql.append("concat_ws(': ', servicechecks.name, servicechecks.hostname) description, ");
		else if ( rc.isUseServiceCheckLabel() )
			sql.append("servicechecks.name description, ");
		else if ( rc.isUseDescriptionLabel() )
			sql.append("servicechecks.description description, ");
		else
			throw new RuntimeException("Unknown label for report configuration: '" + rc.getLabel() + "'");
		
		sql.append("servicechecks.hostname, servicechecks.name, ");
		sql.append("servicechecks.most_recent ");
		sql.append("from servicechecks ");
		sql.append("where hostname = :hostName ");
		sql.append("and name = :name ");
		sql.append("order by description ");
		
		Map <String, String> params = new HashMap<String, String>();
		params.put("name", key.getName());
		params.put("hostName", key.getHostName());
		
		// Execute query to get service checks
		return getJdbcTemplate().query(sql.toString(), new ServiceCheckMapper(), params);
	}
	
	/**
	 * Called by JUnit
	 */
	public List<ServiceCheck> listServiceChecksByKeywords(ReportCriteria rc, List <String> keywords) {
		
		
		StringBuffer sql = new StringBuffer();
		sql.append("select servicechecks.id, ");
		
		if ( rc.isUseHostNameServiceCheckLabel() )
			sql.append("concat_ws(': ', servicechecks.name, servicechecks.hostname) description, ");
		else if ( rc.isUseServiceCheckLabel() )
			sql.append("servicechecks.name description, ");
		else if ( rc.isUseDescriptionLabel() )
			sql.append("servicechecks.description description, ");
		else
			throw new RuntimeException("Unknown label for report configuration: '" + rc.getLabel() + "'");
		
		sql.append("servicechecks.hostname, servicechecks.name, ");
		sql.append("servicechecks.most_recent ");
		sql.append("from servicechecks ");
		sql.append("where ");
		boolean firstLike = true;
		for ( String keyword : keywords ) {

			// Append "and" between like 
			if ( firstLike )
				firstLike = false;
			else
				sql.append("and ");
			
			// Append keyword
			sql.append("keywords like '%,");
			sql.append(keyword);
			sql.append(",%' ");
		}
		sql.append("order by description");
		
		// Execute query to get service checks tagged with the specified keywords
		return getJdbcTemplate().query(sql.toString(), new ServiceCheckMapper());
	}
	
	/**
	 * 
	 */
	public WeeklyServiceAvailability getWeeklyAvailabilityForServiceCheck(ReportCriteria rc, List <Integer> serviceCheckIds) {
		
		StringBuffer sql = new StringBuffer();
		sql.append("select sum(seconds_ok + seconds_warning + seconds_unknown) as ok, ");
		sql.append("sum(seconds_critical_scheduled) as scheduled, ");
		sql.append("sum(seconds_critical - seconds_critical_scheduled) as unscheduled, ");
		sql.append("sum(seconds_ok + seconds_warning + seconds_unknown + seconds_critical) as total ");
		sql.append("from service_availability_hourly_summary ");
		sql.append("where servicecheck ");
		generateInList(sql, serviceCheckIds);
		sql.append(" and ");
		sql.append("CONVERT_TZ(start_datetime,'+00:00','SYSTEM') between :reportStartDateString and :reportEndDateString ");
		
		Map <String, Date> params = new HashMap<String, Date>();
		params.put("reportStartDateString", rc.getReportStartDate());
		params.put("reportEndDateString", rc.getReportEndDate());
		
		// Execute query to get service checks tagged with the specified keywords
		return getJdbcTemplate().queryForObject(sql.toString(), new AvailabilitySummaryMapper(), params);
	}
	
	/**
	 * 
	 */
	public List<WeeklyServiceAvailability> getDailyAvailabilityForServiceCheck(ReportCriteria rc, List <Integer> serviceCheckIds) {
		
		StringBuffer sql = new StringBuffer();
		sql.append("select week(CONVERT_TZ(start_datetime,'+00:00','SYSTEM')) as week, dayofweek(CONVERT_TZ(start_datetime,'+00:00','SYSTEM')) as dow, ");
		sql.append("sum(seconds_ok + seconds_warning + seconds_unknown) as ok, ");
		sql.append("sum(seconds_critical_scheduled) as scheduled, ");
		sql.append("sum(seconds_critical - seconds_critical_scheduled) as unscheduled, ");
		sql.append("sum(seconds_ok + seconds_warning + seconds_unknown + seconds_critical) as total ");
		sql.append("from service_availability_hourly_summary ");
		sql.append("where servicecheck ");
		generateInList(sql, serviceCheckIds);
		sql.append(" and ");
		sql.append("CONVERT_TZ(start_datetime,'+00:00','SYSTEM') between :reportStartDateString and :reportEndDateString ");
		sql.append("group by week, dow");
		
		Map <String, Date> params = new HashMap<String, Date>();
		params.put("reportStartDateString", rc.getReportStartDate());
		params.put("reportEndDateString", rc.getReportEndDate());
		
		// Execute query to get service checks tagged with the specified keywords
		return getJdbcTemplate().query(sql.toString(), new AvailabilityDetailMapper(), params);		
	}
	
	/**
	 * 
	 */
	private class AvailabilitySummaryMapper implements ParameterizedRowMapper<WeeklyServiceAvailability> {

		public WeeklyServiceAvailability mapRow(ResultSet rs, int rowNum) throws SQLException {
			
			WeeklyServiceAvailability summary = new WeeklyServiceAvailability();
			summary.setOk(rs.getDouble("ok"));
			summary.setScheduled(rs.getDouble("scheduled"));
			summary.setUnscheduled(rs.getDouble("unscheduled"));
			summary.setTotal(rs.getDouble("total"));
			
			return summary;
		}
	}
	
	/**
	 * 
	 */
	private class AvailabilityDetailMapper implements ParameterizedRowMapper<WeeklyServiceAvailability> {

		public WeeklyServiceAvailability mapRow(ResultSet rs, int rowNum) throws SQLException {
			
			WeeklyServiceAvailability detail = new WeeklyServiceAvailability();
			detail.setOk(rs.getDouble("ok"));
			detail.setScheduled(rs.getDouble("scheduled"));
			detail.setUnscheduled(rs.getDouble("unscheduled"));
			detail.setTotal(rs.getDouble("total"));

			detail.setDayOfWeek(rs.getInt("dow"));
			
			return detail;
		}
	}		
	 
	/**
	 * 
	 */
	private class ServiceCheckMapper implements ParameterizedRowMapper<ServiceCheck> {

		public ServiceCheck mapRow(ResultSet rs, int rowNum) throws SQLException {
			
			ServiceCheck sc = new ServiceCheck();
			sc.setId(rs.getInt("id"));
			sc.setDescription(rs.getString("description"));
			sc.setMostRecent(rs.getByte("most_recent") == (byte)1);
			sc.setName(rs.getString("name"));
			sc.setHostName(rs.getString("hostname"));
			
			return sc;
		}
	}
	 
	/**
	 * 
	 */
	private class ServiceCheckKeyMapper implements ParameterizedRowMapper<ServiceCheckKey> {

		public ServiceCheckKey mapRow(ResultSet rs, int rowNum) throws SQLException {
			
			ServiceCheckKey key = new ServiceCheckKey();
			key.setHostName(rs.getString("hostName"));
			key.setName(rs.getString("name"));
			
			return key;
		}
	}		
}
