package com.altinity.opsview.odw.reports.test;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.altinity.opsview.odw.model.ServiceCheck;
import com.altinity.opsview.odw.model.ServiceCheckGroup;
import com.altinity.opsview.odw.model.WeeklyServiceAvailability;
import com.altinity.opsview.odw.reports.ReportCriteria;


public class TestDetailDAO extends TestBase {
	
	/**
	 * 
	 */
	public void testAvailabilityDetail() {
		
		// Setup report criteria
		ReportCriteria rc = new ReportCriteria();
		rc.setReportId(2);
		
		try {
			
			SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd");
			Date dataStartDate = formatter.parse("2007-11-05");
			rc.setReportStartDate(dataStartDate);
			
			Calendar cal = Calendar.getInstance();
			cal.setTime(dataStartDate);
			cal.add(Calendar.WEEK_OF_YEAR, 1);
			rc.setReportEndDate(cal.getTime());
		}
		catch(ParseException e) {
			System.out.println("Error attempting to parse date");
			e.printStackTrace();
		}		
		
		// Check service checks are found with the specified keywords
		List <String> keywords = reportsDAO.listKeywordsForReportId(rc);
		List <ServiceCheck> serviceChecks = odwDAO.listServiceChecksByKeywords(rc, keywords);
		
		// Generate map of servicecheck hostname/name key to servicecheck ids
		Map <String,ServiceCheckGroup> keyedSCs = new HashMap<String, ServiceCheckGroup>();
		for ( ServiceCheck sc : serviceChecks ) {
			
			// Get the hostname/name key for this servicecheck
			String key = sc.getName() + sc.getHostName();
			
			// Get the corresponding list for this key
			ServiceCheckGroup group = keyedSCs.get(key);
			
			// Create the list if it doesn't
			// already exist
			if ( group == null ) {
				
				group = new ServiceCheckGroup();
				keyedSCs.put(key, group);
			}
			
			// Add servicecheck to group
			group.addServiceCheck(sc);
		}
		
		// Now query availability for each servicecheck
		for ( String key : keyedSCs.keySet() ) {
			
			ServiceCheckGroup group = keyedSCs.get(key);
			
			// Confirm the group has a most recent
			assertNotNull(group.getText());
			assertNotSame("", group.getText());
			
			// Get the list of daily availability for this servicecheck
			List<WeeklyServiceAvailability> summaries = odwDAO.getDailyAvailabilityForServiceCheck(rc, group.getIds());
			
			// Check core values
			for ( WeeklyServiceAvailability summary : summaries ) {
				double ok = summary.getOk();
				double sched = summary.getScheduled();
				double unSched = summary.getUnscheduled();
				double total = summary.getTotal();
				
				assertEquals(total, ok + sched + unSched);
				
				// Computed values
				double percentOk = summary.getPercentOk();
				double percentSched = summary.getPercentScheduled();
				double percentUnscheduled = summary.getPercentUnscheduled();
				
				assertEquals(100d, percentOk + percentSched + percentUnscheduled);		
			}
		}		
	}	
}
